/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package com.iver.cit.gvsig.fmap.drivers.wcs;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.drivers.RemoteServiceDriver;
import com.iver.cit.gvsig.fmap.layers.WCSParameterModel;

import es.uji.lsi.wcs.XmlWcsParsing.AxisDescription;
import es.uji.lsi.wcs.XmlWcsParsing.CoverageOffering;
import es.uji.lsi.wcs.XmlWcsParsing.GMLEnvelopeWithTimePeriod;
import es.uji.lsi.wcs.XmlWcsParsing.Interval;
import es.uji.lsi.wcs.client.ServerErrorResponseException;
import es.uji.lsi.wcs.client.ServerOutOfOrderException;
import es.uji.lsi.wcs.client.WCSClient;

/**
 * Driver between the FMap and WCSClient
 * 
 * Controlador entre FMap y WCSClient
 * 
 * @author jaume
 *
 */
public class FMapWCSDriver extends RemoteServiceDriver  {
	private WCSClient wcs = null;
	
	/**
	 * Returns the string "WCSDriver", which is the driver's name.
	 * 
	 * Devuelve "WCSDriver", el nombre del driver.
	 * @return String 
	 */
	public String getName() { return "WCSDriver"; }
	
	/**
	 * Sets the server that we want to connect to.
	 * 
	 * Establece el servidor al que se quiere conectar.
	 * 
	 * @param host
	 */
	public void setHost(String host){
		super.setHost(host);
		setServiceName("WCS"); 
		wcs = new WCSClient(host);
	}
	
	/**
	 * Sends the GetCapabilities operation to the server.
	 * 
	 * Enva la operacin GetCapabilities al servidor.
	 * 
	 * @throws ServerOutOfOrderException 
	 */
	public void getCapabilities() throws ServerOutOfOrderException{
		wcs.getCapabilities(); 
	}

	/**
	 * Sends the DescribeCoverage operation to the server.
	 * 
	 * Enva la operacin DescribeCoverage
	 */
	public void describeCoverage(){
		wcs.describeCoverage();
	}
	
	/**
	 * Sends the GetCoverage operation.
	 * 
	 * Enva la operacin GetCoverage.
	 * @throws IOException 
	 */
	public File getCoverage() throws ServerErrorResponseException, IOException{
		return wcs.getCoverage();
		
	}

	/**
	 * Returns the server's name. The string is extracted from the GetCapabilities
	 * response.
	 * 
	 * Devuelve el nombre del servidor indicado por ste.
	 * 
	 * @return String
	 */
	public String getServerName(){
		return wcs.getTheCapabilities().getServiceName();
	}
	
	/**
	 * Returns a human-readable string containing the server's name.
	 * 
	 * Devuelve el nombre legible del servidor devuelto por ste.
	 * 
	 * @return String
	 */
	public String getLabel(){
		return wcs.getTheCapabilities().getServiceLabel();
	}
	
	/**
	 * Returns a string containing the server's WCS version number.
	 * 
	 * Devuelve el nmero de versin WCS del servidor
	 * 
	 * @return String
	 */
	public String getVersion(){
		return wcs.getTheCapabilities().getVersion();
	}
	
	/**
	 * Returns name and description of the server. It is supposed to be used
	 * as the source of the abstract field in your application's interface.
	 * 
	 * Devuelve nombre y descripcin (abstract) del servidor.
	 * 
	 * @return String
	 */
	public String getDescription(){
		String name = wcs.getTheCapabilities().getServiceName();
		String description = wcs.getTheCapabilities().getServiceDescription();
		
		if ( (name == null) && (description == null) ) return "Sin descripcin";
		else if ( (name != null) && (description == null) ) return name;
		else if ( (name == null) && (description != null) ) return description;
		else if ( (name != null) && (description != null) ) return name+"\n\n"+description;
		else return null;
	}
	
	/**
	 * Returns the update sequence value.
	 * 
	 * Devuelve el valor de update sequence del servidor
	 * 
	 * @return String
	 */
	public String getUpdateSequence(){
		return wcs.getTheCapabilities().getUpdateSequence();
	}
	
	/**
	 * Returns a list containing the formats supported by the coverage
	 * 
	 * Devuelve una lista de formatos soportados por la cobertura.
	 * 
	 * @param coverage
	 * @return ArrayList
	 */
	public ArrayList getFormats(String coverage){
		return wcs.getCoverageDescription(coverage).getSupportedFormats().getFormats();
	}
	
	/**
	 * Returns the metadata link.
	 * 
	 * Devuelve el enlace a los metadatos.
	 * 
	 * @return String
	 */
	public String getMetadataLink(){
		return wcs.getTheCapabilities().getServiceMetadatalink();
	}
	
	/**
	 * Returns the responsible party value.
	 * 
	 * Devuelve el valor para responsible party.
	 * 
	 * @return String
	 */
	public String getResponsibleParty(){
		return wcs.getTheCapabilities().getServiceResponsibleParty();
	}
	
	/**
	 * Returns a list containing the names of the coverages located at the server.
	 * 
	 * Devuelve una lista de nombres de coberturas
	 * 
	 * @return String[]
	 */
	public String[] getCoverageNames(){
		return wcs.getTheCapabilities().getCoverageNames();
	}
	
	/**
	 * Returns the number of coverages at the server.
	 * 
	 * Devuelve el numero de coberturas en el servidor
	 * 
	 * @return int
	 */
	public int getNumOfCoverages(){
		return getCoverageNames().length;
	}

	/** 
	 * Returns the details of the coverage given by the coverage's name.
	 * 
	 * Devuelve los detalles de la cobertura en CoverageOffering
	 * @param nomCobertura
	 * @return CoverageOffering (es.uji.lsi.wcs.XmlParsing.CoverageOffering)
	 */
	public CoverageOffering getCoverageDetails(String nomCobertura){
		return wcs.getCoverageDescription(nomCobertura);
	}
	
	/**
	 * Establishes the connection to the WCS server. Connecting to a WCS is
	 * an abstraction, it actually sends a GetCapabilities and a general 
	 * DescribeCoverage request (not a coverage-specific DescribeCoverage request)
	 * to read the necessary data for building further GetCoverage requests.
	 * 
	 * 
	 * La conexin a un servidor WCS es una abstraccin de dos operaciones:
	 * GetCapabilities y DescribeCoverage. Con ellas, se obtienen los datos
	 * necesarios para poder operar con el servidor.
	 * 
	 * 
	 * @throws IOException, DriverException.
	 */
	public void connect() throws IOException, DriverException {
		wcs = new WCSClient(getHost());
		try {
			wcs.getCapabilities();
			wcs.describeCoverage();
			setConnected(true);
		} catch (ServerOutOfOrderException e) {
			throw new DriverException(e);
		}
	}
	/**
	 * No close operation is needed since WCS service it is a non-session based
	 * protocol. So, this does nothing and you can omit it.
	 * 
	 * La operacin close no es necesaria en un servicio WCS ya que este se implementa
	 * sobre un protocolo no orientado a sesiones. As que esto no hace nada y puede
	 * ser omitido.
	 * 
	 */
	public void close() {
		// No fa res, tal volta this= null?
	}

	/**
	 * Returns the full extent for a specific SRS given by the second 
	 * argument srs of the specific coverage given by the coverage's name
	 * in the first argument. You likely preffer to use getFullExtent(String, String)
	 * 
	 * Devuelve la extensin mxima para un SRS especfico dado en el segundo
	 * argumento de una cobertura especifica dada por su nombre en el primer
	 * argumento. Probablemente prefiera usar getFullExtent(String, String)
	 * 
	 * @param coverageName
	 * @param srs
	 * @throws DriverException
	 * @throws IOException
	 */
	private Rectangle2D getBoundingBoxExtensionMaxima(String coverageName, String srs) throws IOException, DriverException {
		// TODO Has de considerar el cas per a quan el SRS no vinga als RequestResponse 
		// ni cap altre...
		// IDEA: agafar  algun dels de CoverageOfferingBrief i aplicar la transformaci.
		CoverageOffering co = wcs.getCoverageDescription(coverageName);
		
		if (!isConnected())
		    connect();
		if (co.getDomainSet() != null) {
			if (co.getDomainSet().getSpatialDomain() != null){
				ArrayList gewtpList = co.getDomainSet().getSpatialDomain().getGMLEnvelopeWithTimePeriodList();
				Iterator it = gewtpList.iterator();
				while (it.hasNext()){
					GMLEnvelopeWithTimePeriod gewtp = (GMLEnvelopeWithTimePeriod) it.next();
					if (gewtp.getSRSName().equals(srs)) {
						return new Rectangle2D.Double(
								gewtp.getMinX(),
								gewtp.getMinY(),
								gewtp.getMaxX()-gewtp.getMinX(),
								gewtp.getMaxY()-gewtp.getMinY());
					}
				}
			}
		}
		
		// Si llego aqu es porque no ha habido ninguno. Tengo que transformar de las del BoundingBox.
		
		return null;
	}

	/**
	 * Sends the GetCoverage query args using a string containing list of 
	 * &-separated pair-values (name=value)
	 * 
	 * Enva los valores de la consulta GetCoverage en forma cadena de pares 
	 * 'nombre=valor' separados por '&'.
	 */
	public void setGetCoverageParams(String query) {
		wcs.setGetCoverageParams(query);
		
	}
	

	/**
	 * Returns the label of an specific coverage given by the coverage name
	 * 
	 * Obtiene la etiqueta de una cobertura especfica
	 * 
	 * @param nombre de la cobertura (string)
	 * @return string
	 */
	public String getLabel(String coverageName) {
		CoverageOffering co = getCoverageDetails(coverageName);
		return co.getCoverageOfferingBrief().getLabel();
	}

	/**
	 * Returns the coverage's MAX extent from the server.
	 * 
	 * Obtiene la MXIMA extensin de la cobertura de el servidor.
	 * @return Rectangle2D
	 * @throws DriverException
	 * @throws IOException
	 */
	public Rectangle2D getFullExtent(String coverageName, String srs) throws IOException, DriverException {
		return getBoundingBoxExtensionMaxima(coverageName, srs);
	}

	/**
	 * Returns an Arraylist containing the parameters of the coverage given by
	 * the coverage name. The list contains a set of Parametro objects.
	 * 
	 * Devuelve los parmetros soportados por la cobertura especificada por su nombre.
	 * La lista contiene un juego de objetos Parametro.
	 * 
	 * @return ArrayList de Parametro
	 */
	public ArrayList getParameters(String coverageName) {
		CoverageOffering co = getCoverageDetails(coverageName);
		ArrayList lista_descripciones =	co.getRangeSet().getAxisDescriptionList();
		ArrayList lista_parametros = new ArrayList();
		if (lista_descripciones!= null){
			Iterator it = lista_descripciones.iterator();
			while (it.hasNext()){
				AxisDescription ad = ((AxisDescription) it.next());
				WCSParameterModel p = new WCSParameterModel();
				p.setName(ad.getName());
				p.setType(ad.getValuesType());
				if (p.getType().equals("singleValue")){
						p.setLista_valores_simples(ad.getSingleValues());
				}
				lista_parametros.add(p);
				if (p.getType().equals("interval")){
						Interval interv = ad.getInterval();
						p.setMin(interv.getMin());
						p.setMax(interv.getMax());
						p.setRes(interv.getRes());
				}
			}
		}
		return lista_parametros;
	}
	
	/**
	 * Returns the max resolution of a specific coverage given by the coverage's name.
	 * 
	 * Obtiene la resolucin mxima de una cobertura especfica
	 * @param nombre de la cobertura (string)
	 * @return double
	 */
	/*public double getMaxResolution(String coverageName) {
		CoverageOffering co = getCoverageDetails(coverageName);
		return co.getDomainSet().getSpatialDomain().getMaxRes();
	}*/
	public Point2D getMaxResolution(String coverageName) {
		CoverageOffering co = getCoverageDetails(coverageName);
		return co.getDomainSet().getSpatialDomain().getMaxRes();
	}


	/**
	 * Returns an ArrayList containing a set of Strings with the coverage's SRSs.
	 * 
	 * Obtiene un ArrayList con los SRS de una cobertura especfica
	 * @param Nombre de la cobertura (string)
	 * @return ArrayList
	 */
	public ArrayList getSRSs(String coverageName) {	
		// TODO Si el servidor da algun CRS en RequestResponse se usar esta respuesta
		// 		en caso contrario, se usarn el de la cobertura en SpatialDomain.
		//		Esto est por revisar, ya que debera tener inteligencia para
		//		transformar entre sistemas de proyecciones diferentes
		
		CoverageOffering co = getCoverageDetails(coverageName);
		return co.getDomainSet().getSpatialDomain().getGMLEnvelopeNames();
	}

	/**
	 * Returns a String containing a description of an specific coverage.
	 * 
	 * Obtiene la descripcin de una cobertura especfica
	 * @param Nombre de la cobertura (string)
	 * @return string
	 */
	public String getCoverageDescription(String coverageName) {
		CoverageOffering co = getCoverageDetails(coverageName);
		return co.getCoverageOfferingBrief().getDescription();
	}

	/**
	 * Returns an ArrayList containing strings for the time positions of an
	 * specific coverage given by the coverage's name.
	 * 
	 * Obtiene la lista de tiempos de una cobertura especfica
	 * @param nombre de la cobertura (string)
	 * @return ArrayList
	 */
	public ArrayList getTimes(String coverageName) {
		CoverageOffering co = getCoverageDetails(coverageName);
		if (co.getDomainSet().getTemporalDomain()!=null){
			return co.getDomainSet().getTemporalDomain().getGMLTimePositionList();
		}
		return null;
	} 
}
