package es.uji.lsi.wcs.XmlWcsParsing;

import java.io.File;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Hashtable;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Esta clase representa un XMLNode simplificado
 * Contiene una lista de subnodos y una lista de atributos.
 * Tambin tiene una cadena.
 * 
 * Soporta la lectura y escritura de y desde un fichero XML.
 * Tambin soporta la lectura desde Internet
 *
 * Modificado por jaume
 */ 

public class XMLNode {
   
   // subnodos
   private Vector subNodes = new Vector();
   // atributos
   private Hashtable attr = new Hashtable();
   private String nodeName;
   private String text = null;
   // los nombres de los atributos
   private Vector attrKeys = new Vector();
   // cabecera XML
   private String header = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";

   /**
    * Constructor, lee de un fichero
    * 
    * @author jaume - jaume.dominguez@iver.es
    *
    */ 
   public XMLNode(File file) throws Exception {
      this(factory.newDocumentBuilder().parse(file));
   }
   
   /**
    * Constructor. Usando url.openStream() se puede usar
    * para leer info descargada de internet.
    * @param inputstream
    * @throws Exception
    */
   public XMLNode(InputStream inputstream) throws Exception{
      this(factory.newDocumentBuilder().parse(inputstream));
   }

   /**
    * Contructor, constructor desde un documento DOM
    * 
    * @author jaume - jaume.dominguez@iver.es
    *
    */ 
   public XMLNode(Document dom) throws Exception {
      this((Element)dom.getFirstChild());
   }

   /**
    *  Contructor, crea un nodo con su nombre
    */
   public XMLNode(String name) throws Exception {
      nodeName = name;
   }

   /**
    * Contructor, crea un nodo con su nombre y el texto
    */ 
   public XMLNode(String name, String text) throws Exception {
      nodeName = name;
      this.text = text;
   }

   /**
    * Contructor, desde un elemento DOM
    */
   public XMLNode(Element dom) throws Exception {
      nodeName = dom.getNodeName();
      NamedNodeMap map = dom.getAttributes();
      for (int i = 0; i < map.getLength(); i++) {
         Node att = map.item(i);
         addAtrribute(att.getNodeName(), att.getNodeValue());
      }
      NodeList nodeList = dom.getChildNodes();
      for (int i = 0; i < nodeList.getLength(); i++) {
         Node sub = nodeList.item(i);
         if (sub.getNodeType() == Node.ELEMENT_NODE) {
            addSubNode(new XMLNode((Element)sub));
         }
         else if (sub.getNodeType() == Node.TEXT_NODE) {
            String s = sub.getNodeValue().trim();
            if (s.length() > 0) {
               if (text != null) {
                  throw new Exception("XMLNode '" + nodeName + "' has 2 Textblocks");
               }
               text = s;
            }
         }
      }
   }

   public void setText(String s) {
      text = s;
   }

   public void addSubNode(XMLNode s) {
      subNodes.add(s);
   }

   public void addAtrribute(String name, String value) throws Exception {
      if (attr.containsKey(name)) {
         throw new Exception(
               "XMLNode '" + nodeName + "' already contains Attribute '" + name + "'");
      }
      attr.put(name, value);
      attrKeys.add(name);
   }

   public int getNumSubNodes() {
      return subNodes.size();
   }

   public String getName() {
      return nodeName;
   }

   public String getText() {
      return text;
   }

   public XMLNode getSubNode(int index) {
      return (XMLNode)subNodes.get(index);
   }

   public Vector getAttributeNames() {
       return attrKeys;
   }
   
   public void write(Writer wr) throws Exception {
      wwrite("", wr);
   }

   /**
    * Escribe el cdigo XML de este objeto con su identacin
    */ 
   private void wwrite(String pre, Writer wr) throws Exception {
      wr.write(pre + "<" + nodeName);
      for (int i = 0; i < attrKeys.size(); i++) {
         String name = (String)attrKeys.get(i);
         String val = (String)attr.get(name);
         wr.write(" " + name + "='" + val + "'");
      }
      if (getNumSubNodes() == 0 && text == null) {
         wr.write("/>\n");
      }
      else {
         wr.write(">");
         if (text != null) {
            wr.write(text);
         }
         if (getNumSubNodes() > 0) {
            wr.write("\n");
            for (int i = 0; i < subNodes.size(); i++) {
               if (getSubNode(i) != null) {
                  getSubNode(i).wwrite(pre + "  ", wr);
               }
            }
            wr.write(pre + "</" + nodeName + ">\n");
         }
         else {
            wr.write("</" + nodeName + ">\n");
         }
      }
   }
   
   


   public String getAttribute(String key) {
      return (String)attr.get(key);
   }

   public double getDoubleAttribute(String key) {
      return Double.parseDouble((String)attr.get(key));
   }

   public boolean getBoolAttribute(String key) {
      if (!hasAttribute(key)) {
         return false;
      }
      return new Boolean((String)attr.get(key)).booleanValue();
   }

   public int getIntAttribute(String key) {
      return Integer.parseInt((String)attr.get(key));
   }

   public boolean hasAttribute(String key) {
      return attr.containsKey(key);
   }

   // escribe al fichero
   public void write(File f) throws Exception {
      FileWriter fw = new FileWriter(f);
      fw.write(header);
      write(fw);
      fw.flush();
      fw.close();
   }


   public String toString() {
      StringWriter wr = new StringWriter();
      try {
         write(wr);
      }
      catch (Exception e) {
         System.err.println("Exception in StringWriter " + e);
         e.printStackTrace();
         System.exit(1);
      }
      return wr.toString();
   }

   static DocumentBuilderFactory factory;
   static {
      factory = DocumentBuilderFactory.newInstance();
      factory.setValidating(false);
      factory.setNamespaceAware(false);
      factory.setIgnoringComments(true);
   }

   public void setHeader(String header) {
      this.header = header;
   }

   public static void main(String []args) throws Exception {
      XMLNode node=new XMLNode(new File(args[0]));
      System.out.println(node);
   }
}
