/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.andami;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

import javax.swing.SwingUtilities;

import org.apache.log4j.Logger;

import com.iver.andami.messages.Messages;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.plugins.Extension;
import com.iver.andami.plugins.ExtensionDecorator;
import com.iver.andami.plugins.PluginClassLoader;
import com.iver.andami.ui.mdiFrame.MDIFrame;
import com.iver.andami.ui.mdiFrame.MainFrame;
import com.iver.andami.ui.mdiManager.MDIManager;
import com.iver.utiles.XMLEntity;


/**
 * Clase que proporciona servicios a los plugins.
 *
 * @author Fernando Gonzlez Corts
 */
public class PluginServices {
	private static Logger logger = Logger.getLogger(PluginServices.class.getName());
	private PluginClassLoader loader;
	private PropertyResourceBundle resourceBundle;
	private XMLEntity persistentXML;

	/**
	 * Crea un nuevo PluginServices
	 *
	 * @param loader ClassLoader del plugin
	 */
	public PluginServices(PluginClassLoader loader) {
		this.loader = loader;
	}

	/**
	 * Devuelve el mensaje en el idioma del locale actual del texto
	 * correspondiente a la clave que se pasa como parmetro
	 *
	 * @param key Clave del texto que se quiere obtener
	 *
	 * @return Texto en el idioma que toca o la propia clave si no se encuentra
	 * 		   en el fichero
	 */
	public String getText(String key) {
		if (resourceBundle == null) {
			return key;
		}

		if (key == null) {
			return null;
		}

		try {
			return resourceBundle.getString(key);
		} catch (MissingResourceException e) {
			logger.warn(getPluginName()+": "+Messages.getString("PluginServices.No_se_encontro_la_traduccion_para") + " " + key);

			return key;
		}
	}

	/**
	 * Obtiene el classloader del plugin
	 *
	 * @return Returns the loader.
	 */
	public PluginClassLoader getClassLoader() {
		return loader;
	}

	/**
	 * Obtiene el nombre del plugin
	 *
	 * @return String
	 */
	String getPluginName() {
		return loader.getPluginName();
	}

	/**
	 * Establece el paquete de traducciones del objeto
	 * pluginservices asociado a un plugin
	 *
	 * @param name nombre del paquete
	 * @param locale locale para obtener los recursos del paquete
	 */
	void setResourceBundle(String name, Locale locale) {
		try {
			resourceBundle = (PropertyResourceBundle) ResourceBundle.getBundle(name,
					locale, loader);
		} catch (MissingResourceException e) {
			logger.error(Messages.getString("PluginServices.No_se_encontro_el_recurso_de_traducciones") +
					" '"+name+"' en "+getPluginName(), e);
		}
	}

	/**
	 * Obtienen una referencia al PluginServices del plugin cuyo nombre se pasa
	 * como parmetro
	 *
	 * @param pluginClassInstance Instancia de una clase propia del plugin a
	 * 		  cuyos servicios se quiere acceder
	 *
	 * @return Objeto PluginServices asociado al plugin
	 *
	 * @throws RuntimeException Si el parmetro no es un objeto cargado desde un plugin
	 */
	public static PluginServices getPluginServices(Object pluginClassInstance) {
		try {
			PluginClassLoader loader = (PluginClassLoader) pluginClassInstance.getClass()
																			  .getClassLoader();

			return Launcher.getPluginServices(loader.getPluginName());
		} catch (ClassCastException e) {
			/* throw new RuntimeException(
				"Parameter is not a plugin class instance"); */
		    return null;
		}
	}

	/**
	 * Obtienen una referencia al PluginServices del plugin cuyo nombre se pasa
	 * como parmetro
	 *
	 * @param pluginName Instancia de una clase propia del plugin a cuyos
	 * 		  servicios se quiere acceder
	 *
	 * @return Objeto PluginServices asociado al plugin
	 */
	public static PluginServices getPluginServices(String pluginName) {
		return Launcher.getPluginServices(pluginName);
	}

	/**
	 * Obtiene una referencia al gestor de ventanas
	 *
	 * @return MDIManager
	 */
	public static MDIManager getMDIManager() {
		return Launcher.getFrame().getMDIManager();
	}

	/**
	 * Obtiene una referencia al marco principal de la
	 * aplicacin
	 *
	 * @return MainFrame
	 */
	public static MainFrame getMainFrame() {
		return Launcher.getFrame();
	}

	/**
	 * Obtiene una referencia a la instancia de la extensin
	 * cuya clase se pasa como parmetro
	 *
	 * @param extensionClass Clase de la extensin cuya instancia se quiere obtener
	 *
	 * @return Instancia de la extensin o null en caso de que no haya
	 * una extensin con esa clase
	 */
	public static Extension getExtension(Class extensionClass) {
		ExtensionDecorator extAux = (ExtensionDecorator) Launcher.getClassesExtensions().get(extensionClass);
		return extAux.getExtension(); 
		// return (Extension) Launcher.getClassesExtensions().get(extensionClass);
		
	}

	/**
	 * Gets a reference to the Extension Decorator with support for several more options
	 * @param extensionClass
	 * @return
	 */
	public static ExtensionDecorator getDecoratedExtension(Class extensionClass) {
		return (ExtensionDecorator) Launcher.getClassesExtensions().get(extensionClass);
	}
	
	/**
	 * Obtiene un iterador sobre las extensiones
	 *
	 * @return Iterator
	 */
	public static Iterator getExtensions() {
		return Launcher.getClassesExtensions().values().iterator();
	}

	/**
	 * Obtiene una traduccin de un plugin
	 *
	 * @param pluginObject Objeto cargado desde un plugin
	 * @param key Nombre de la clave cuyo valor se quiere obtener
	 *
	 * @return El valor, si existe. La clave si no existe. Si el objeto que
	 * se pasa como parmetro no ha sido cargado desde un plugin
	 * se devuelve la clave.
	 */
	public static String getText(Object pluginObject, String key) {
		try {
			PluginServices ps = getPluginServices(pluginObject);
			if (ps == null) return key;
			return ps.getText(key);
		} catch (RuntimeException e) {
			logger.error("GetText: Clave = " + key + ". ", e);

			return key;
		}
	}

	/**
	 * Establece los datos del plugin que debern persistir entre ejecuciones
	 * en formato xml
	 *
	 * @param entity DOCUMENT ME!
	 */
	public void setPersistentXML(XMLEntity entity) {
		persistentXML = entity;
	}

	/**
	 * Obtiene un objeto que representa la persistencia del plugin en formato
	 * xml.
	 *
	 * @return Devuelve null hasta que se invoca el mtodo setPersistentXML
	 */
	public XMLEntity getPersistentXML() {
		if (persistentXML == null){
			persistentXML = new XMLEntity();
		}
		return persistentXML;
	}

	/**
	 * Aade un listener a un popup menu registrado en el
	 * config.xml de algn plugin
	 *
	 * @param name Nombre del men contextual
	 * @param c Componente que desplegar el men cuando
	 * se haga click con el botn derecho
	 * @param listener Listener que se ejecutar cuando se 
	 * seleccione cualquier entrada del men
	 *
	 * @throws RuntimeException Si la interfaz no est preparada
	 * todava. Slo puede darse durante el arranque
	 */
	public void addPopupMenuListener(String name, Component c,
		ActionListener listener) {
		MDIFrame frame = Launcher.getFrame();

		if (frame == null) {
			throw new RuntimeException("MDIFrame not loaded yet");
		}

		frame.addPopupMenuListener(name, c, listener, loader);
	}

	/**
	 * Obtiene una referencia al directorio del plugin
	 *
	 * @return File
	 */
	public File getPluginDirectory() {
		return new File(Launcher.getPluginsDir() + File.separator +
			getPluginName());
	}

	/**
	 * Ejecuta una tarea en segundo plano, dejando a la 
	 * interfaz responder pero inhibiendo los eventos
	 *
	 * @param r Tarea a ejecutar
	 *
	 * @return Thread en el que se ejecuta la tarea
	 */
	public static Thread backgroundExecution(Runnable r) {
		Thread t = new Thread(new RunnableDecorator(r));
		t.start();

		return t;
	}

	public static void closeApplication(){
	    Launcher.closeApplication();
	}
	
	/**
	 * DOCUMENT ME!
	 *
	 * @author Fernando Gonzlez Corts
	 */
	private static class RunnableDecorator implements Runnable {
		private Runnable r;

		/**
		 * Crea un nuevo RunnableDecorator.
		 *
		 * @param r DOCUMENT ME!
		 */
		public RunnableDecorator(Runnable r) {
			this.r = r;
		}

		/**
		 * @see java.lang.Runnable#run()
		 */
		public void run() {
			/* try {
				SwingUtilities.invokeAndWait(new Runnable() {
						public void run() {
							getMDIManager().setWaitCursor();
						}
					});
			} catch (InterruptedException e) {
				NotificationManager.addWarning(Messages.getString("PluginServices.No_se_pudo_poner_el_reloj_de_arena"),
					e);
			} catch (InvocationTargetException e) {
				NotificationManager.addWarning(Messages.getString("PluginServices.No_se_pudo_poner_el_reloj_de_arena"),
					e);
			} */

			try {
			    SwingUtilities.invokeAndWait(new Runnable() {
                    public void run() {
            			try{
            			    r.run();
            			} catch(RuntimeException e){
            				NotificationManager.addError(Messages.getString("PluginServices.Bug_en_el_codigo"), e);
            			} catch (Error e){
            				NotificationManager.addError(Messages.getString("PluginServices.Error_grave_de_la_aplicacin"), e);
            			}
                    }
                });
			} catch (InterruptedException e) {
				NotificationManager.addWarning(Messages.getString("PluginServices.No_se_pudo_poner_el_reloj_de_arena"),
					e);
			} catch (InvocationTargetException e) {
				NotificationManager.addWarning(Messages.getString("PluginServices.No_se_pudo_poner_el_reloj_de_arena"),
					e);
			}

			/* try {
				SwingUtilities.invokeAndWait(new Runnable() {
						public void run() {
							getMDIManager().restoreCursor();
						}
					});
			} catch (InterruptedException e) {
				NotificationManager.addWarning(Messages.getString("PluginServices.No_se_pudo_restaurar_el_cursor_del_raton"),
					e);
			} catch (InvocationTargetException e) {
				NotificationManager.addWarning(Messages.getString("PluginServices.No_se_pudo_restaurar_el_cursor_del_raton"),
					e);
			} */
		}
	}
}
