/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io;

import org.cresques.px.PxRaster;

import java.io.IOException;

import java.util.TreeMap;


/**
 * Clase abstracta de la que heredan los drivers de escritura. Tiene los
 * mtodos abstractos que debe implementar cualquier driver de escritura
 * y las funcionalidades y opciones soportadas comunes a todos ellos.
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public abstract class GeoRasterWriter {
    private static TreeMap supportedExtensions = null;
    protected String outfilename = null;
    protected String infilename = null;
    protected int sizeWindowX = 0;
    protected int sizeWindowY = 0;
    protected int ulX = 0;
    protected int ulY = 0;
    protected PxRaster currentRaster;
    protected IDataWriter dataWriter = null;
    protected int nBands = 0;
    protected String ident = null;
    protected String driver = null;

    /**
     * Registra un formato de escritura
     * @param ext        Extensin del fichero registrado
     * @param clase        Clase que maneja el formato registrado
     */
    public static void registerWriterExtension(String ext, Class clase) {
        if (supportedExtensions == null) {
            supportedExtensions = new TreeMap();
        }

        ext = ext.toLowerCase();
        System.out.println("Write RASTER: extension '" + ext + "' supported.");
        supportedExtensions.put(ext, clase);
    }

    /**
     * Devuelve el nmero de drivers soportados
     * @return Nmero de drivers soportados
     */
    public static int getNDrivers() {
        return supportedExtensions.size();
    }

    /**
     * Devuelve la lista de extensiones de escritura soportadas
     * @return
     */
    public static String[] getDriversExtensions() {
        String[] list = new String[GeoRasterWriter.getNDrivers()];

        if (GeoRasterWriter.getNDrivers() > 0) {
            list[0] = supportedExtensions.firstKey().toString();

            for (int i = 1; i < list.length; i++) {
                list[i] = supportedExtensions.lastKey().toString();
            }
        }

        return list;
    }

    /**
     * Devuelve el identificador del driver
     * @return        Identificador del driver
     */
    public String getIdent() {
        return ident;
    }

    /**
     * Obtiene el nombre del driver.
     * @return        Nombre del driver
     */
    public String getDriverName() {
        return driver;
    }

    /**
     * Asigna propiedades al driver a partir de un vector de
     * strings donde cada elemento tiene la estructura de
     * propiedad=valor.
     * @param props        Propiedades
     */
    public abstract void setProps(String[] props);

    /**
     * Realiza la funcin de compresin a partir de un GeoRasterFile.
     * @throws IOException
     */
    public abstract void fileWrite() throws IOException;

    /**
     * Realiza la funcin de compresin a partir de los datos pasados por el cliente.
     * @throws IOException
     */
    public abstract void dataWrite() throws IOException;

    /**
     * Cierra el driver
     */
    public abstract void writeClose();

    /**
     * Devuelve la configuracin de la ventana de dialogo
     * para las propiedades del driver de escritura que se est tratando.
     * @return Texto XML con las propiedades
     */
    public abstract String getXMLPropertiesDialog();
}
