/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.raster;


/**
 * Clase base para los filtros de transparencia en sus diferentes tipos
 * de datos.
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public abstract class TransparencyFilter extends RasterFilter {
    //Parmetros del filtro
    protected int[][] rangesR;
    protected int[][] rangesG;
    protected int[][] rangesB;

    /**
     * Valor de transparencia
     */
    public int alpha = 0x10;

    /**
     * Color en la banda del rojo para la transparencia
     */
    public int transparencyColorRed = 0xff;

    /**
     * Color en la banda del verde para la transparencia
     */
    public int transparencyColorGreen = 0xff;

    /**
     * Color en la banda del azul para la transparencia
     */
    public int transparencyColorBlue = 0xff;

    /**
     * Constructor
     *
     */
    public TransparencyFilter() {
        super();
    }

    /**
     * Obtiene parmetros del filtro desde la tabla Hash
     */
    public void pre() {
        //Obtencin de parmetros comunes a todos
        this.rangesR = (int[][]) params.get("red");
        this.rangesG = (int[][]) params.get("green");
        this.rangesB = (int[][]) params.get("blue");
        this.alpha = ((Integer) params.get("alpha")).intValue();
        this.transparencyColorRed = ((Integer) params.get("transparencyRed")).intValue();
        this.transparencyColorGreen = ((Integer) params.get("transparencyGreen")).intValue();
        this.transparencyColorBlue = ((Integer) params.get("transparencyBlue")).intValue();
    }

    /**
     * Para dos intervalos dados, devuelve true si el intervalo B est dentro del A y
     * false si no lo est
     * @param intervalA intervalo A
     * @param intervalB intervalo B
     * @return true si el intervalo B est dentro del A y false si no lo est
     */
    private boolean compareIntervals(int[][] intervalA, int[][] intervalB) {
        if ((intervalA != null) && (intervalB != null)) {
            boolean equalInterval = false;

            for (int i = 0; i < intervalB.length; i++) {
                for (int j = 0; j < intervalA.length; j++) {
                    if ((intervalB[i][0] >= intervalA[i][0]) &&
                            (intervalB[i][1] <= intervalA[i][1])) {
                        equalInterval = true;

                        break;
                    }
                }

                if (!equalInterval) {
                    return false;
                }
            }

            return true;
        }

        if ((intervalA == null) && (intervalB == null)) {
            return true;
        }

        return false;
    }

    /**
     * Devuelve true si el filtro pasado por parmetro es equivalente a este
     * y false si no lo es.
     * @param filter        Filtro a comparar
     * @return        true si son equivalentes y false si no lo son
     */
    public boolean isEquivalent(TransparencyFilter filter) {
        if ((alpha != filter.alpha) ||
                (transparencyColorRed != filter.transparencyColorRed) ||
                (transparencyColorGreen != filter.transparencyColorGreen) ||
                (transparencyColorBlue != filter.transparencyColorBlue)) {
            return false;
        }

        boolean equalRange = false;
        equalRange = compareIntervals((int[][]) params.get("red"),
                                      (int[][]) filter.params.get("red"));

        if (!equalRange) {
            return false;
        }

        equalRange = false;
        equalRange = compareIntervals((int[][]) params.get("green"),
                                      (int[][]) filter.params.get("green"));

        if (!equalRange) {
            return false;
        }

        equalRange = false;
        equalRange = compareIntervals((int[][]) params.get("blue"),
                                      (int[][]) filter.params.get("blue"));

        if (!equalRange) {
            return false;
        }

        return true;
    }

    /**
     * Obtiene los rangos de transparencia en la banda  del rojo
     * @return rangos de transparencia en la banda del rojo
     */
    public int[][] getRangeR() {
        return this.rangesR;
    }

    /**
     * Obtiene los rangos de transparencia en la banda  del verde
     * @return rangos de transparencia en la banda del verde
     */
    public int[][] getRangeG() {
        return this.rangesG;
    }

    /**
     * Obtiene los rangos de transparencia en la banda  del azul
     * @return rangos de transparencia en la banda del azul
     */
    public int[][] getRangeB() {
        return this.rangesB;
    }
}
