/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import org.cresques.io.CXMLParser;
import org.cresques.io.GeoRasterWriter;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


/**
 * Panel que maneja los eventos de las propiedades de escritura del panel de un
 * tipo de raster concreto. Estan propiedades son obtenidas desde un texto en XML
 * que es generado por el driver. Este XML se pasa desde aqu al panel que
 * contendr los elementos para su generacin dinmica. Dependiendo del tipo de
 * driver el panel generado (DataInputProps) ser diferente.
 * @author Nacho Brodin <brodin_ign@gva.es>
 */
public class SavePropsSetupPanel extends JPanel implements ActionListener,
                                                           ChangeListener,
                                                           MouseListener,
                                                           FocusListener,
                                                           KeyListener {
    final private static long serialVersionUID = -3370601314380922368L;
    private DataInputProps saveRaster = null;
    private SaveRasterPropsDialogPanel dialogPanel = null;
    private String fName = null;
    private GeoRasterWriter writer = null;
    private String formato = null;
    private String[] properties = null;

    /**
     * Constructor
     *
     */
    public SavePropsSetupPanel() {
        super();
    }

    /**
     * Asigna el formato del fichero del que se quieren ver las propiedades
     * @param formato
     */
    public void setFormat(String formato) {
        this.formato = formato;
        initialize();
    }

    /**
     * Asigna el GeoRasterFile que corresonde al driver
     * @param writer
     */
    public void setWriter(GeoRasterWriter writer) {
        this.writer = writer;
        initialize();
    }

    /**
     * Inicializacin de la ventana
     *
     * @return void
     */
    void initialize() {
        int widthPanel = 350;
        int heightPanel = 210;

        //Obtenemos el ancho y alto del panel interior del XML
        String xml = writer.getXMLPropertiesDialog();

        if (xml != null) {
            CXMLParser parser = new CXMLParser(xml);
            parser.getValue("window", true);

            String w = parser.getAttr("panel", "sizex");
            String h = parser.getAttr("panel", "sizey");

            if ((w != null) && !w.equals("")) {
                widthPanel = Integer.parseInt(w);
            }

            if ((h != null) && !h.equals("")) {
                heightPanel = Integer.parseInt(h);
            }
        }

        this.setPreferredSize(new Dimension(widthPanel, heightPanel)); //AnchoxAlto  panel interior
        this.setBounds(0, 0, 300, 250);
        this.setLayout(null);
        this.add(getSaveParameters(), null);

        //Aadimos gestin de eventos        
        if (saveRaster.getCombos() != null) {
            for (int i = 0; i < saveRaster.getCombos().length; i++)
                saveRaster.getCombos()[i].addActionListener(this);
        }

        if (saveRaster.getSliders() != null) {
            for (int i = 0; i < saveRaster.getSliders().length; i++)
                saveRaster.getSliders()[i].getSlider().addChangeListener(this);
        }

        if (saveRaster.getChecks() != null) {
            for (int i = 0; i < saveRaster.getChecks().length; i++)
                saveRaster.getChecks()[i].addActionListener(this);
        }
    }

    /**
     * Asigna un valor al panel padre
     * @param dialogPanel
     */
    public void setDialogPanel(SaveRasterPropsDialogPanel dialogPanel) {
        this.dialogPanel = dialogPanel;
    }

    /**
     * Obtiene el Panel de Controles interior
     * @return
     */
    public DataInputProps getSaveParameters() {
        if (saveRaster == null) {
            saveRaster = new DataInputProps(writer);
        }

        return saveRaster;
    }

    /**
     * Controla cuando se cumplen todos los requisitos en el formulario para
     * poder activar el botn de aceptar.
     *
     */
    private void enableButtons() {
        if (dialogPanel != null) {
            dialogPanel.getAcceptButton().setEnabled(true);
        }
    }

    /**
     * Devuelve un vector de Strings en el que cada elemento es un par propiedad=valor
     * con las propiedades del driver seleccionado
     * @return Vector de propiedades
     */
    public String[] getProperties() {
        return properties;
    }

    /**
     * Obtiene el nuevo vector de propiedades
     */
    public void stateChanged(ChangeEvent e) {
        properties = saveRaster.getElements();
    }

    /**
     * Obtiene el nuevo vector de propiedades
     */
    public void actionPerformed(ActionEvent e) {
        properties = saveRaster.getElements();
    }

    /**
     * Devuelve el nombre del fichero seleccionado
     * @return Nombre del fichero seleccionado
     */
    public String getFileName() {
        return fName;
    }

    /**
     * Controla la activacin y desactivacin de los botones
     */
    public void mouseClicked(MouseEvent e) {
        enableButtons();
    }

    /**
     * Controla la activacin y desactivacin de los botones
     */
    public void focusLost(FocusEvent e) {
        enableButtons();
    }

    /**
     * Controla la activacin y desactivacin de los botones
     */
    public void keyTyped(KeyEvent e) {
        enableButtons();
    }

    /**
     * Controla la activacin y desactivacin de los botones
     */
    public void keyPressed(KeyEvent e) {
        enableButtons();
    }

    /**
     * Controla la activacin y desactivacin de los botones
     */
    public void keyReleased(KeyEvent e) {
        enableButtons();
    }

    /**
     * Unused. Only for compilant purposes
     */
    public void mouseExited(MouseEvent e) {
    }

    /**
     * Unused. Only for compilant purposes
     */
    public void mouseReleased(MouseEvent e) {
    }

    /**
     * Unused. Only for compilant purposes
     */
    public void mouseEntered(MouseEvent e) {
    }

    /**
     * Unused. Only for compilant purposes
     */
    public void mousePressed(MouseEvent e) {
    }

    /**
     * Unused. Only for compilant purposes
     */
    public void focusGained(FocusEvent e) {
    }
}
