package com.iver.cit.gvsig.fmap.edition;

import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;

import java.io.IOException;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.HashMap;


/**
 * DOCUMENT ME!
 *
 * @author Vicente Caballero Navarro
 */
public class DefaultEditableFeatureSource implements EditableFeatureSource {
	private ExpansionFile expansionFile;
	private OriginalFeatureAdapter ofa;
	private BitSet delgeometries = new BitSet();
	private HashMap relations = new HashMap();

	/**
	 * Crea un nuevo AbstractEditableFeatureSource.
	 *
	 * @param ef DOCUMENT ME!
	 * @param ofa DOCUMENT ME!
	 */
	public DefaultEditableFeatureSource(ExpansionFile ef,
		OriginalFeatureAdapter ofa) {
		expansionFile = ef;
		this.ofa = ofa;
	}

	/**
	 * Mtodo invocado cuando se comienza la edicin, para poner en marcha las
	 * estructuras de datos necesarias para la misma, notificar al servidor en
	 * protocolos en los que sea necesario, ...
	 * @throws EditionException
	 */
	public void startEdition() throws EditionException {
		ofa.startEdition();
	}

	/**
	 * Invocado cuando termina la edicin. En funcin de la clase concreta que
	 * implemente este mtodo se generar el fichero con los resultados de la
	 * edicin, se realizar una transaccin a la base de datos, etc.
	 * @throws EditionException
	 */
	public void stopEdition() throws EditionException {
		ofa.stopEdition();
	}

	/**
	 * Devuelve el nmero de geometrias que hay actualmente en edicin.
	 *
	 * @return Nmero de geometras.
	 */
	public int getGeometryCount() {
		return expansionFile.getGeometryCount();
	}

	/**
	 * Si el ndice se corresponde a una geometria de las originales de la capa
	 * en edicin y no ha sido modificada ni eliminada devuelve la geometria
	 * original. Si ha sido modificada debera de buscar en el fichero de
	 * expansin y si ha sido eliminada debera devolver null
	 *
	 * @param index ndice de la geometra.
	 *
	 * @return Geometra.
	 *
	 * @throws IOException
	 * @throws DriverIOException
	 */
	public IGeometry getGeometry(int index)
		throws IOException, DriverIOException {
		Integer integer = new Integer(index);

		if (!relations.containsKey(integer)) {
			if (delgeometries.get(index)) {
				return null;
			} else {
				return ofa.getGeometry(index);
			}
		} else {
			int num = ((Integer) relations.get(integer)).intValue();

			return expansionFile.getGeometry(num);
		}
	}

	/**
	 * Aade una geometria al fichero de expansin y guarda la correspondencia
	 * en una tabla asociada.
	 *
	 * @param g geometra a guardar.
	 *
	 * @throws DriverIOException
	 * @throws IOException
	 */
	public void addGeometry(IGeometry g) throws DriverIOException, IOException {
		int virtualIndex = ofa.getGeometryCount() + expansionFile.getGeometryCount();
		int pos = expansionFile.addGeometry(g);
		relations.put(new Integer(virtualIndex), new Integer(pos));
	}

	/**
	 * Elimina una geometria. Si es una geometra original de la capa en
	 * edicin se marca como eliminada (haya sido modificada o no). Si es una
	 * geometra aadida posteriormente se invalida en el fichero de
	 * expansin, para que una futura compactacin termine con ella.
	 *
	 * @param index ndice de la geometra.
	 */
	public void removeGeometry(int index) {
		Integer integer = new Integer(index);

		if (!relations.containsKey(integer)) {
			delgeometries.set(index, true);
		} else {
			int num = ((Integer) relations.get(integer)).intValue();
			expansionFile.invalidateGeometry(num);
		}
	}

	/**
	 * Si se intenta modificar una geometra original de la capa en edicin se
	 * aade al fichero de expansin y se registra la posicin en la que se
	 * aadi. Si se intenta modificar una geometria que se encuentra en el
	 * fichero de expansin (por ser nueva o original pero modificada) se
	 * invoca el mtodo modifyGeometry y se actualiza el ndice de la
	 * geometria en el fichero.
	 *
	 * @param index DOCUMENT ME!
	 * @param g DOCUMENT ME!
	 *
	 * @throws IOException
	 */
	public void modifyGeometry(int index, IGeometry g)
		throws IOException {
		Integer integer = new Integer(index);

		if (!relations.containsKey(integer)) {
			relations.put(integer, new Integer(expansionFile.addGeometry(g)));
		} else {
			int num = ((Integer) relations.get(integer)).intValue();
			num = expansionFile.modifyGeometry(num, g);
			relations.put(integer, new Integer(num));
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public OriginalFeatureAdapter getOriginalFeatureAdapter() {
		return ofa;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.edition.EditableFeatureSource#compact()
	 */
	public void compact() {
		expansionFile.compact(relations);
	}
}
