/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.FMap;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.operations.Cancellable;

import com.iver.utiles.XMLEntity;

import org.cresques.cts.ICoordTrans;

import org.cresques.geo.Projected;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;


/**
 * Interfaz que tienen que implementar todas las capas.
 */
public interface FLayer extends Projected {
	/**
	 * Obtiene una representacin de la coleccin de capas de forma recursiva
	 *
	 * @return XMLEntity.
	 * @throws XMLException
	 */
	XMLEntity getXMLEntity() throws XMLException;

	/**
	 * Inserta las propiedades del XMLEntity al objeto actual.
	 *
	 * @param xml XMLEntity
	 *
	 * @throws XMLException
	 */
	void setXMLEntity(XMLEntity xml) throws XMLException;

	/**
	 * Inserta las propiedades del XMLEntity al objeto actual.
	 *
	 * @param xml XMLEntity
	 *
	 * @throws XMLException
	 */
	void setXMLEntity03(XMLEntity xml) throws XMLException;

	/**
	 * Pone la capa actual a activa o inactiva segn el boolean que se pasa
	 * como parmetro.
	 *
	 * @param selected activa.
	 */
	void setActive(boolean selected);

	/**
	 * Devuelve true si la capa esta activa.
	 *
	 * @return activa.
	 */
	boolean isActive();

	/**
	 * Inserta un nombre a la capa.
	 *
	 * @param name nombre.
	 */
	void setName(String name);

	/**
	 * Devuelve el nombre de la capa.
	 *
	 * @return nombre de la capa.
	 */
	String getName();

	/**
	 * Realiza las operaciones de inicializacin de la capa. El mtodo es
	 * invocado una nica vez durante la vida de la capa y justo antes de
	 * visualizar la capa
	 *
	 * @throws DriverIOException
	 */
	void load() throws DriverIOException;

	/**
	 * Pone la capa en modo visible o no visible.
	 *
	 * @param visibility visibilidad.
	 */
	void setVisible(boolean visibility);

	/**
	 * Devuelve true si la capa es visible.
	 *
	 * @return visibilidad.
	 */
	boolean isVisible();

	/**
	 * Devuelve el FLayers padre de la capa.
	 *
	 * @return FLayers padre de la capa.
	 */
	public FLayers getParentLayer();

	/**
	 * Devuelve el FMap al que est aadida la capa o null si la capa no ha
	 * sido aadida a ningn FMap
	 *
	 * @return FMap
	 */
	public FMap getFMap();

	/**
	 * Inserta el FLayers padre de la capa.
	 *
	 * @param root capa padre.
	 */
	public void setParentLayer(FLayers root);

	/**
	 * Obtiene la extensin completa de la capa
	 *
	 * @return FullExtent.
	 *
	 * @throws DriverException
	 */
	Rectangle2D getFullExtent() throws DriverException;

	/**
	 * Dibuja la capa
	 *
	 * @param image Imagen utilizada para acelerar el dibujado en pantalla.
	 * @param g Graphics2D sobre el que dibujar.
	 * @param viewPort Propiedades de la vista.
	 * @param cancel PAra poder cancelar el dibujado.
	 *
	 * @throws DriverException
	 */
	void draw(BufferedImage image, Graphics2D g, ViewPort viewPort,
		Cancellable cancel,double scale) throws DriverException;

	/**
	 * Dibuja la capa
	 *
	 * @param g Graphics2D de la impresora sobre el que dibujar.
	 * @param viewPort Propiedades de la vista.
	 * @param cancel
	 *
	 * @throws DriverException
	 */
	void print(Graphics2D g, ViewPort viewPort, Cancellable cancel,double scale)
		throws DriverException;

	/**
	 * Inserta las coordenadas de transformacin.
	 *
	 * @param ct Coordenadas de transformacin.
	 */
	void setCoordTrans(ICoordTrans ct);

	/**
	 * Devuelve las coordenadas de transformacin.
	 *
	 * @return Coordenadas de transformacin.
	 */
	ICoordTrans getCoordTrans();

	/**
	 * Aade un listener LayerListener a la lista de listeners.
	 *
	 * @param o Listener.
	 *
	 * @return True si es correcta la insercin del listener.
	 */
	public boolean addLayerListener(LayerListener o);

	/**
	 * Borra de la lista el LayerListener que se pasa como parmetro.
	 *
	 * @param o Listener.
	 *
	 * @return True si es correcto el borrado del listener.
	 */
	public boolean removeLayerListener(LayerListener o);
	
	public boolean isWithinScale(double scale);
	
	
	/**
	 * La capa no se visualiza si est por debajo de esa escala
	 * @return la escala minima de visualizacin
	 */
	public double getMinScale();

	/**
	 * La capa no se visualiza si est por encima de esa escala
	 * @return la escala mxima de visualizacin
	 */
	public double getMaxScale();
	
	public void setMinScale(double minScale);
	public void setMaxScale(double maxScale);
}
