/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.sql.SQLException;
import java.sql.Types;
import java.util.TreeMap;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.cresques.cts.IProjection;

import com.hardcode.driverManager.Driver;
import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.driverManager.DriverManager;
import com.hardcode.gdbms.engine.customQuery.QueryManager;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.NoSuchTableException;
import com.hardcode.gdbms.engine.data.edition.DataWare;
import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ProgressListener;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.drivers.DBLayerDefinition;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.RasterDriver;
import com.iver.cit.gvsig.fmap.drivers.VectorialDatabaseDriver;
import com.iver.cit.gvsig.fmap.drivers.VectorialDriver;
import com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver;
import com.iver.cit.gvsig.fmap.drivers.VectorialJDBCDriver;
import com.iver.cit.gvsig.fmap.drivers.WithDefaultLegend;
import com.iver.cit.gvsig.fmap.operations.arcview.ArcJoin;
import com.iver.cit.gvsig.fmap.rendering.LegendFactory;
import com.iver.cit.gvsig.fmap.rendering.VectorialLegend;


/**
 * Crea un adaptador del driver que se le pasa como parmetro en los mtodos
 * createLayer. Si hay memoria suficiente se crea un FLyrMemory que pasa todas
 * las features del driver a memoria
 */
public class LayerFactory {
	private static Logger logger = Logger.getLogger(LayerFactory.class.getName());
	private static String driversPath = "../FMap 03/drivers";
	private static DriverManager driverManager;

	private static DataSourceFactory dataSourceFactory;
	
	/**
	 * Map en el que se guarda para cada fuente de datos aadida al sistema, el
	 * adaptador que la maneja. Ha de ser un TreeMap ya que esta clase define
	 * la igualdad entre las claves a traves del mtodo equals de las mismas.
	 * Los objetos FileSource, DBSource y WFSSource tienen definido el mtodo
	 * equals de forma que devuelven true cuando dos objetos apuntan a la
	 * misma fuente de datos
	 */
	private static TreeMap sourceAdapter;

	/*
	 * Crea un RandomVectorialFile con el driver que se le pasa como parmetro
	 * y guardndose el nombre del fichero para realizar los accesos, la capa
	 * tendr asociada la proyeccin que se pasa como parametro tambin
	 *
	 * @param layerName Nombre de la capa.
	 * @param driverName Nombre del driver.
	 * @param f fichero.
	 * @param proj Proyeccin.
	 *
	 * @return FLayer.
	 * @throws DriverException
	 *
	 * @throws DriverException
	 * @throws DriverIOException
	 */
	public static FLayer createLayer(String layerName, String driverName,
		File f, IProjection proj) throws DriverException  {
		//Se obtiene el driver que lee
		DriverManager dm = getDM();

		try {
			Driver d = dm.getDriver(driverName);

			if (d instanceof VectorialFileDriver) {
				return createLayer(layerName, (VectorialFileDriver) d, f, proj);
			} else if (d instanceof RasterDriver) {
				return createLayer(layerName, (RasterDriver) d, f, proj);
			}
		} catch (DriverLoadException e) {
			throw new DriverException(e);
		}

		return null;
	}

	/**
	 * Crea un RandomVectorialFile con el driver que se le pasa como parmetro
	 * y guardndose el nombre del fichero para realizar los accesos, la capa
	 * tendr asociada la proyeccin que se pasa como parametro tambin
	 *
	 * @param layerName Nombre del Layer.
	 * @param d VectorialAdapter.
	 * @param f Fichero.
	 * @param proj Proyeccin.
	 *
	 * @return FLayer creado.
	 *
	 * @throws DriverException
	 */
	public static FLayer createLayer(String layerName, VectorialFileDriver d,
		File f, IProjection proj) throws DriverException {
		//TODO Comprobar si hay un adaptador ya
		VectorialFileAdapter adapter = new VectorialFileAdapter(f);
		adapter.setDriver((VectorialDriver) d);

		FLyrVect capa = new FLyrVect();
		capa.setName(layerName);
		try{
		//TODO Meter esto dentro de la comprobacin de si hay memoria
		if (false) {
		} else {
			capa.setSource(adapter);
			capa.setProjection(proj);
		}
		//try {
			
			// Le asignamos tambin una legenda por defecto acorde con
			// el tipo de shape que tenga. Tampoco s si es aqu el
			// sitio adecuado, pero en fin....
			if (d instanceof WithDefaultLegend) {
				WithDefaultLegend aux = (WithDefaultLegend) d;
				adapter.start();
				capa.setLegend((VectorialLegend) aux.getDefaultLegend());
				adapter.stop();
			} else {
				capa.setLegend(LegendFactory.createSingleSymbolLegend(
						capa.getShapeType()));
			}
		/*} catch (FieldNotFoundException e) {
			//Esta no puede saltar
		} catch (DriverIOException e) {
			throw new DriverException(e);
		}*/
		}catch (Exception e) {
		/*	JOptionPane.showMessageDialog(null,"Se ha producido un error cargando la capa '" 
					+layerName+"'\n El recurso asociado a la capa es : "
					+f.getAbsolutePath()+"\n Esta capa ser eliminada del proyecto.");
			*/
			//capa.setBroken(true);	
			//return capa;
			throw new DriverException(e);
		}

		return capa;
	}

    public static FLayer createLayer(String layerName, VectorialDriver d,
            IProjection proj) throws DriverException {
            //TODO Comprobar si hay un adaptador ya
            VectorialDefaultAdapter adapter = new VectorialDefaultAdapter();
            adapter.setDriver((VectorialDriver) d);

            FLyrVect capa = new FLyrVect();
            capa.setName(layerName);

            capa.setSource(adapter);
            capa.setProjection(proj);

            try {
                
                // Le asignamos tambin una legenda por defecto acorde con
                // el tipo de shape que tenga. Tampoco s si es aqu el
                // sitio adecuado, pero en fin....
                if (d instanceof WithDefaultLegend) {
                    WithDefaultLegend aux = (WithDefaultLegend) d;
                    adapter.start();
                    capa.setLegend((VectorialLegend) aux.getDefaultLegend());
                    adapter.stop();
                } else {
                    capa.setLegend(LegendFactory.createSingleSymbolLegend(
                            capa.getShapeType()));
                }
            } catch (FieldNotFoundException e) {
                //Esta no puede saltar
            } catch (DriverIOException e) {
                throw new DriverException(e);
            }

            return capa;
        }

	/**
	 * Crea una capa Raster a partir del nombre driver, fichero y proyeccin.
	 *
	 * @param layerName Nombre de la capa.
	 * @param d RasterDriver.
	 * @param f Fichero.
	 * @param proj Proyeccin.
	 *
	 * @return Nueva capa de tipo raster.
	 *
	 * @throws DriverIOException
	 */
	public static FLyrRaster createLayer(String layerName, RasterDriver d,
		File f, IProjection proj) throws DriverException {
		RasterAdapter adapter = new RasterFileAdapter(f);
		adapter.setDriver(d);

		FLyrRaster capa = new FLyrRaster();
		capa.setName(layerName);

		//TODO Meter esto dentro de la comprobacin de si hay memoria
		if (false) {
		} else {
			capa.setSource(adapter);
			capa.setProjection(proj);
			try {
				capa.load();
			} catch (DriverIOException e) {
				throw new DriverException(e);
			}
		}

		return capa;
	}

	/**
	 * Crea una capa Raster a partir del nombre driver, fichero y proyeccin y 
	 * coordenadas de georeferenciacin. Esta funcin es para georeferenciar 
	 * capas raster. Para imagenes que no tienen georeferenciacin hay que asignarle
	 * una temporal, normalmente a partir de la vista activa.
	 *
	 * @param layerName Nombre de la capa.
	 * @param d RasterDriver.
	 * @param f Fichero.
	 * @param proj Proyeccin.
	 * @param extent Extent de la vista activa
	 *
	 * @return Nueva capa de tipo raster.
	 *
	 * @throws DriverIOException
	 */
	public static FLyrRaster createLayer(String layerName, RasterDriver d,
		File f, IProjection proj, ViewPort vp) throws DriverException {
		RasterAdapter adapter = new RasterFileAdapter(f);
		adapter.setDriver(d);
		
		FLyrRaster capa = new FLyrRaster();
		capa.setName(layerName);
		
		//TODO Meter esto dentro de la comprobacin de si hay memoria
		if (false) {
		} else {
			capa.setSource(adapter);
			capa.setProjection(proj);
			//capa.setTempExtent(vp);
			try {
				capa.load();
			} catch (DriverIOException e) {
				throw new DriverException(e);
			}
		}

		return capa;
	}
	
	
    public static FLayer createArcSDELayer(String layerName,VectorialDatabaseDriver driver, IProjection proj) {
            //throw new UnsupportedOperationException();
            FLyrVect layer=new FLyrVect();
            VectorialAdapter adapter=new VectorialDBAdapter();
            adapter.setDriver(driver);
            
            layer.setName(layerName);
            layer.setSource(adapter);
            layer.setProjection(proj);
            try {
                if (driver instanceof WithDefaultLegend) {
                    WithDefaultLegend aux = (WithDefaultLegend) driver;
                    adapter.start();
                    layer.setLegend((VectorialLegend) aux.getDefaultLegend());
                    adapter.stop();
                } else { 
                    layer.setLegend(LegendFactory.createSingleSymbolLegend(
                            layer.getShapeType()));
                }
            } catch (FieldNotFoundException e) {
                throw new UnsupportedOperationException(e.getMessage());
            } catch (DriverIOException e) {
                throw new UnsupportedOperationException(e.getMessage());
            } catch (DriverException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            
            try {
                layer.setLegend(LegendFactory.createSingleSymbolLegend(
                        layer.getShapeType()));
            } catch (FieldNotFoundException e) {
                e.printStackTrace();
            } catch (DriverException e) {
                e.printStackTrace();
            }
            return layer;
        }

	/**
	 * Crea un RandomVectorialWFS con el driver que se le pasa como parmetro y
	 * guardndose la URL del servidor que se pasa como parmetro
	 *
	 * @param driver
	 * @param host
	 * @param port
	 * @param user
	 * @param password
	 * @param dbName
	 * @param tableName
	 * @param proj
	 *
	 * @return Capa creada.
	 *
	 * @throws UnsupportedOperationException
	 */
	public static FLayer createLayer(VectorialDatabaseDriver driver,
		String host, int port, String user, String password, String dbName,
		String tableName, IProjection proj) {
		throw new UnsupportedOperationException();
	}
	public static FLayer createDBLayer(VectorialDatabaseDriver driver,String layerName, IProjection proj)
	{
	    
	        
		FLyrVect capa = new FLyrVect();
		
		capa.setName(layerName);
		VectorialDBAdapter dbAdapter = new VectorialDBAdapter();
		dbAdapter.setDriver(driver);			
	
		capa.setSource(dbAdapter);
		capa.setProjection(proj);
		try {
			if (driver instanceof WithDefaultLegend) {
				WithDefaultLegend aux = (WithDefaultLegend) driver;
				dbAdapter.start();
	            capa.setLegend((VectorialLegend) aux.getDefaultLegend());
				dbAdapter.stop();
			} else { 
				capa.setLegend(LegendFactory.createSingleSymbolLegend(
						capa.getShapeType()));
			}
        } catch (FieldNotFoundException e) {
            throw new UnsupportedOperationException(e.getMessage());
        } catch (DriverException e) {
            throw new UnsupportedOperationException(e.getMessage());
        }

		return capa;


	    	
		}

	/**
	 * @param driver
	 * @param layerName
	 * @param object
	 * @return
	 * @throws SQLException
	 * @throws DriverIOException
	 * @throws IOException
	 * @throws DriverLoadException
	 * @throws com.hardcode.gdbms.engine.data.driver.DriverException
	 * @throws NoSuchTableException
	 * @throws ClassNotFoundException
	 * @throws 
	 */
	public static FLayer createDisconnectedDBLayer(VectorialJDBCDriver driver, String layerName, IProjection proj, ProgressListener listener) throws SQLException, IOException, DriverIOException, DriverLoadException, NoSuchTableException, com.hardcode.gdbms.engine.data.driver.DriverException, ClassNotFoundException {
	    VectorialDisconnectedDBAdapter dbAdapter = new VectorialDisconnectedDBAdapter();
	    dbAdapter.setDriver(driver);
	    DataSource ds = dbAdapter.getRecordset();
	    ds.start();
	    String database = dataSourceFactory.getTempFile();
	    String[] fieldNames = new String[ds.getFieldCount() + 1];
	    System.arraycopy(ds.getFieldNames(), 0, fieldNames, 1, ds.getFieldCount());
	    fieldNames[0] = "the_geom";
	    int[] types = new int[fieldNames.length];
	    types[0] = Types.BINARY;
	    for (int i = 1; i < types.length; i++) {
            types[i] = ds.getFieldType(i-1);
        }
	    String dsName = dataSourceFactory.createTable(database, ds.getPKNames(), fieldNames, types);
	    
        DBLayerDefinition lyrDef = new DBLayerDefinition();
        lyrDef.setTableName(dsName);
        lyrDef.setLayerName(layerName);
        lyrDef.setFieldNames(ds.getFieldNames());
        lyrDef.setFieldGeometry("the_geom");
        lyrDef.setFieldID(ds.getPKNames()[0]);
        lyrDef.setClassToInstantiate("org.hsqldb.jdbcDriver");
        
	    dataSourceFactory.addDBDataSourceByTable(dsName, null, 0, "sa", "", database, dsName, "GDBMS HSQLDB Transactional driver");
	    DataSource local = dataSourceFactory.createRandomDataSource(dsName, DataSourceFactory.MANUAL_OPENING);
	    local.start();
	    DataWare dw = local.getDataWare(DataSourceFactory.DATA_WARE_COHERENT_ROW_ORDER);
	    dw.start();
	    long t1 = System.currentTimeMillis();
	    dw.beginTrans();
	    
	    if (listener == null){
	        listener = new ProgressListener() {
                /**
                 * @see com.iver.cit.gvsig.fmap.ProgressListener#progress(int)
                 */
                public void progress(int n) {
                    //do nothing
                }
            };
	    }
	    
	    for (int i = 0; i < dbAdapter.getShapeCount(); i++) {
	        Value[] row = new Value[ds.getFieldCount() + 1];
	        
	        byte[] bytes = dbAdapter.getShape(i).toWKB();
	        row[0] = ValueFactory.createValue(bytes);
	        	        
	        for (int j = 0; j < ds.getFieldCount(); j++) {
	            row[j+1] = ds.getFieldValue(i, j);                
            }

	        dw.insertFilledRow(row);
	        listener.progress(100 * i / dbAdapter.getShapeCount());
        }
	    
	    long t2 = System.currentTimeMillis();
	    dw.commitTrans();
	    long t3 = System.currentTimeMillis();
	    System.out.println((t2 - t1) + " - " + (t3 - t2));
	    dw.stop();
	    local.stop();
	    ds.stop();
	    VectorialJDBCDriver cacheDriver = (VectorialJDBCDriver) LayerFactory.getDM().getDriver("HSQLDB Driver");
        Class.forName("org.hsqldb.jdbcDriver");

	    cacheDriver.setData(java.sql.DriverManager.getConnection("jdbc:hsqldb:file:" + database, "sa", ""), lyrDef);
	    cacheDriver.setWorkingArea(driver.getWorkingArea());
        return createDBLayer(cacheDriver, layerName, proj);
	}
	
	
	/**
	 * Obtiene el adaptador de un driver si ya se ha aadido el origen de datos
	 * o null si es un origen de datos nuevo
	 *
	 * @param source Adaptador.
	 */
	private static void getAdapter(Source source) {
	}

	/**
	 * Registra la asociacin entre la fuente de datos que se pasa como
	 * parmetro y el VectorialAdapter que se pasa como parmetro, para
	 * satisfacer futuras llamadas a getAdapter
	 *
	 * @param s Adaptador
	 * @param a VectorialAdapter.
	 */
	private static void saveSourceAdapter(Source s, VectorialAdapter a) {
	}

	/**
	 * Crea una FLyrComplexRaster que ataca al driver que se pasa como
	 * parmetro.
	 *
	 * @param driver
	 * @param f
	 * @param proj
	 *
	 * @throws IllegalArgumentException Si se pasa un driver que no implementa
	 * 		   GeorreferencedRasterDriver o NotGeorreferencedRasterDriver
	 */
	public static void createLayer(RasterDriver driver, File f, IProjection proj)
		throws IllegalArgumentException {
	}

	/**
	 * Devuelve el DriverManager.
	 *
	 * @return DriverManager.
	 */
	public static DriverManager getDM() {
		initializeDriverManager();

		return driverManager;
	}

	/**
	 * Inicializa el DriverManager.
	 */
	private static void initializeDriverManager() {
		if (driverManager == null) {
			driverManager = new DriverManager();
			driverManager.loadDrivers(new File(LayerFactory.driversPath));

			Throwable[] failures = driverManager.getLoadFailures();

			for (int i = 0; i < failures.length; i++) {
				logger.error("", failures[i]);
			}

			getDataSourceFactory().setDriverManager(driverManager);
			getDataSourceFactory().initialize();
			QueryManager.registerQuery(new ArcJoin());
		}
	}

	/**
	 * sets drivers Directory
	 *
	 * @param path
	 */
	public static void setDriversPath(String path) {
		LayerFactory.driversPath = path;
		initializeDriverManager();
	}
	
	
	/**
	 * @return Returns the dataSourceFactory.
	 */
	public static DataSourceFactory getDataSourceFactory() {
		if (dataSourceFactory == null){
			dataSourceFactory = new DataSourceFactory();
		}
		return dataSourceFactory;
	}
}
