/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import com.hardcode.driverManager.DriverLoadException;

import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.drivers.DriverAttributes;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.VectorialDriver;
import com.iver.cit.gvsig.fmap.operations.strategies.ShapeInfo;
import com.iver.cit.gvsig.fmap.rendering.indexes.Indexable;

import java.awt.Image;
import java.awt.geom.Rectangle2D;
import java.nio.ByteBuffer;
import java.sql.Types;


/**
 * Clase padre de los adaptadores de los drivers. De momento mantiene solo el
 * ndice creado sobre la capa
 */
public abstract class VectorialAdapter implements ReadableVectorial {
	protected VectorialDriver driver;
	private ShapeInfo shapeInfo;

	/**
	 * Establece el driver sobre el que acta el adaptador
	 *
	 * @param driver
	 */
	public void setDriver(VectorialDriver driver) {
		this.driver = driver;
	}

	/**
	 * Obtiene una referencia al objeto que implementa la interfaz vectorial
	 * con el fin de que las Strategy puedan optimizar en funcin del driver.
	 *
	 * @return VectorialDriver
	 */
	public VectorialDriver getDriver() {
		return driver;
	}

	/**
	 * Crea un ndice para la capa. Se comprueba si el driver implementa
	 * Indexable. Si lo es, se delega la creacin en el driver, si no se crea
	 * un ndice interno con IndexFactory
	 *
	 * @param boundingBox Rectngulo.
	 *
	 * @throws DriverIOException
	 */
	public void createIndex(Rectangle2D boundingBox) throws DriverIOException {
	}

	/**
	 * Mira si la capa tiene un ndice, bien propio del tipo de capa, bien
	 * interno de gvSIG
	 *
	 * @return True si la capa contiene ndice.
	 */
	public boolean hasIndex() {
		//TODO Implementar bien
		return false;
	}

	/**
	 * Si el driver es indexable devuelve el valor de la invocacin a
	 * getIndexFile del driver, si no, devuelve la ruta del fichero de ndices
	 * interno de gvSIG
	 *
	 * @return String.
	 */
	public String getIndexFile() {
		//TODO implementar bien
		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.indexes.Index#openIndexFile(java.lang.String)
	 */
	public void openIndexFile(String file) {
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.rendering.indexes.Index#closeIndexFile()
	 */
	public void closeIndexFile() {
	}

	/**
	 * Devuelve el DataSource a pasrtir del nombre.
	 *
	 * @return DataSource.
	 *
	 * @throws DriverLoadException
	 */
	public abstract DataSource getRecordset()
		throws DriverLoadException;

	/**
	 * Devuelve el ShapeInfo.
	 *
	 * @return Returns the shapeInfo.
	 */
	public ShapeInfo getShapeInfo() {
		return shapeInfo;
	}
    
    

	/**
	 * Inserta el ShapeInfo.
	 *
	 * @param shapeInfo The shapeInfo to set.
	 */
	public void setShapeInfo(ShapeInfo shapeInfo) {
		this.shapeInfo = shapeInfo;
	}

	
	/**
	 * Por defecto devuelve null, y se le pone el icono por defecto.
	 * Si el driver reescribe este mtodo, se usar este icono en el TOC.
	 * 
	 * @return
	 */
	public Image getImageIcon()
	{
	    return null; 
	}
	
	public DriverAttributes getDriverAttributes()
	{
	    if (driver != null)
	        return driver.getDriverAttributes();
	    return null;
	}
    
    /**
     * En la implementacin por defecto podemos hacer que cada
     * feature tenga ID = numero de registro.
     * En el DBAdapter podramos "overrride" este mtodo y poner
     * como ID de la Feature el campo nico escogido en
     * la base de datos.
     * @param numReg
     * @return
     * @throws DriverException
     */
    public IFeature getFeature(int numReg) throws DriverException
    {
        IGeometry geom;
        IFeature feat = null;
        try {
            geom = getShape(numReg);
            DataSource rs = getRecordset();
            Value[] regAtt = new Value[rs.getFieldCount()];
            for (int fieldId=0; fieldId < rs.getFieldCount(); fieldId++ )
            {       
                regAtt[fieldId] =  rs.getFieldValue(numReg, fieldId);
            }
            
            feat = new DefaultFeature(geom, regAtt, numReg + "");
        } catch (DriverIOException e) {
            throw new DriverException(e);
        } catch (DriverLoadException e) {
            throw new DriverException(e);
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            throw new DriverException(e);
        }
        return feat;
    }

}
