/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.operations.strategies;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.operations.Cancellable;

import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import java.util.BitSet;


/**
 * Interfaz estrategia.
 */
public interface Strategy {
	/**
	 * Dibuja la capa vectorial asociada al Strategy en la imagen que se pasa
	 * como parmetro.
	 *
	 * @param image
	 * @param g
	 * @param viewPort
	 * @param cancel
	 *
	 * @throws DriverException
	 */
	void draw(BufferedImage image, Graphics2D g, ViewPort viewPort,
		Cancellable cancel) throws DriverException;

	/**
	 * Dibujar esta Shape en el Graphics con el smbolo que se pasa como
	 * parmetro y despues de aplicarle la transformacin que se pasa tambin
	 * como parmetro. Dibujar la geometria en caso de que la IGeometry
	 * intersecte o est contenida en el rectngulo que se pasa como parmetro
	 *
	 * @param g
	 * @param viewPort
	 * @param cancel
	 *
	 * @throws DriverException
	 */
	void print(Graphics2D g, ViewPort viewPort, Cancellable cancel)
		throws DriverException;

	/**
	 * Recorre las features de la capa vectorial invocando el mtodo visit del
	 * visitador que se pasa como parmetro, que es el que realizar la
	 * operacin relacionada con la geometry
	 *
	 * @param visitor
	 * @param subset
	 *
	 * @throws DriverException
	 * @throws VisitException Si el visitor lanza una excepcin al procesar una
	 * 		   geometra de la capa
	 */
	void process(FeatureVisitor visitor, BitSet subset)
		throws DriverException, VisitException;

	/**
	 * Recorre las features de la capa vectorial invocando el mtodo visit del
	 * visitador que se pasa como parmetro, que es el que realizar la
	 * operacin relacionada con la geometry
	 *
	 * @param visitor
	 *
	 * @throws DriverException
	 * @throws VisitException Si el visitor lanza una excepcin al procesar una
	 * 		   geometra de la capa
	 */
	void process(FeatureVisitor visitor) throws DriverException, VisitException;

	/**
	 * Realiza una query por punto en coordenadas del mundo real de la capa
	 * vectorial asociada a la estrategia
	 *
	 * @param p
	 * @param tolerance
	 *
	 * @return BitSet con los ndices de los registros de la repuesta a la
	 * 		   consulta.
	 *
	 * @throws DriverException
	 */
	FBitSet queryByPoint(Point2D p, double tolerance) throws DriverException;

	/**
	 * Realiza una query por rectngulo en coordenadas del mundo real de la
	 * capa vectorial asociada a la estrategia
	 *
	 * @param rect Rectngulo sobre el que hacer la consulta.
	 *
	 * @return BitSet con los ndices de los registros.
	 *
	 * @throws DriverException
	 */
	FBitSet queryByRect(Rectangle2D rect) throws DriverException;

	/**
	 * Realiza una query por IGeometry en coordenadas del mundo real de la capa
	 * vectorial asociada a la estrategia
	 *
	 * @param g
	 * @param relationship
	 *
	 * @return BitSet con los ndices de los registros.
	 *
	 * @throws DriverException
	 * @throws VisitException
	 */
	FBitSet queryByShape(IGeometry g, int relationship)
		throws DriverException, VisitException;

	/**
	 * Obtiene el rectngulo mnimo que contiene todas las features
	 * seleccionadas o null si no hay ninguna seleccionada
	 *
	 * @return Devuelve el extent de la consulta.
	 */
	Rectangle2D getSelectionBounds();

	/**
	 * Crea un ndice sobre la capa vectorial de la estrategia
	 */
	void createIndex();
}
